'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.encodeAction = encodeAction;
exports.omitActionSignature = omitActionSignature;
exports.verifyAction = verifyAction;
exports.signAction = signAction;
exports.codifyMessage = codifyMessage;

var _jsonStableStringify = require('json-stable-stringify');

var _jsonStableStringify2 = _interopRequireDefault(_jsonStableStringify);

var _constants = require('./constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function encodeAction(action) {
  return (0, _jsonStableStringify2.default)(action);
}

function omitActionSignature(action) {
  var _extends2;

  return _extends({}, action, {
    meta: _extends({}, action.meta, (_extends2 = {}, _defineProperty(_extends2, _constants.META_PUBLIC_KEY, undefined), _defineProperty(_extends2, _constants.META_SIGNATURE, undefined), _extends2))
  });
}

// calls back with true or false, whether the action's embedded signature
// metadata is accurate
function verifyAction(identity, callback, action) {
  var publicKey = action && action.meta && action.meta[_constants.META_PUBLIC_KEY],
      signature = action && action.meta && action.meta[_constants.META_SIGNATURE];

  callback(identity.verifyPublic(encodeAction(omitActionSignature(action)), signature, publicKey));
}

// calls back with the action with signature metadata added
function signAction(identity, callback, action) {
  var _extends3;

  callback(_extends({}, action, {
    meta: _extends({}, action.meta, (_extends3 = {}, _defineProperty(_extends3, _constants.META_PUBLIC_KEY, identity.publicKey), _defineProperty(_extends3, _constants.META_SIGNATURE, identity.sign(encodeAction(action))), _extends3))
  }));
}

// Sign message (must be an array, or it'll be treated as a hex sequence)
function codifyMessage(message) {
  return message.split('').map(function (m) {
    return m.charCodeAt(0);
  });
}